@@ -220,25 +220,15 @@ void _wglGetLastError()
 // Description	    : 
 //---------------------------------------------------------------------------
 /**
-* @fn PrintExtensionError( char* strMsg, ... )
+* @fn PrintExtensionError( char* strMsg )
 * @brief Prints an error about missing OpenGL extensions.
 */ 
-void PrintExtensionError( const char* strMsg, ... )
+void PrintExtensionError( const char* strMsg )
 {
-    fprintf(stderr, 
+    fprintf(stderr,
             "Error: RenderTexture requires the following unsupported "
             "OpenGL extensions: \n");
-    char strBuffer[512];
-    va_list args;
-    va_start(args, strMsg);
-#ifdef _WIN32
-    _vsnprintf( strBuffer, 512, strMsg, args );
-#else
-    vsnprintf( strBuffer, 512, strMsg, args );
-#endif
-    va_end(args);
-    
-    fprintf(stderr, strMsg);
+    fprintf(stderr, "%s", strMsg);
 }
 
 //---------------------------------------------------------------------------
@@ -409,7 +399,6 @@ bool RenderTexture::Initialize(int width, int height,
     _pDisplay = glXGetCurrentDisplay();
     GLXContext context = glXGetCurrentContext();
     int screen = DefaultScreen(_pDisplay);
-    XVisualInfo *visInfo;
     
     GLXFBConfigSGIX *fbConfigs;
     int nConfigs;
@@ -443,28 +432,44 @@ bool RenderTexture::Initialize(int width, int height,
                                                          GLX_RGBA_TYPE, 
                                                          _bShareObjects ? context : NULL, 
                                                          True);
-            break;
+            // [Florian] Try alternative way
+            if(!_hGLContext)
+            {
+                XVisualInfo * visInfo = glXGetVisualFromFBConfig(_pDisplay, fbConfigs[i]);
+
+                if (!visInfo)
+                {
+                    fprintf(stderr,
+                            "RenderTexture Error: glXGetVisualFromFBConfig() failed.\n");
+                }
+                else
+                {
+                    _hGLContext = glXCreateContext(_pDisplay, visInfo,
+                                                   _bShareObjects ? context : NULL, GL_TRUE);
+
+                    XFree(visInfo);
+                }
+            }
+
+            if (_hGLContext)
+            {
+                break;
+            }
         }
     }
     
     if (!_hPBuffer)
     {
-        fprintf(stderr, 
+        fprintf(stderr,
                 "RenderTexture Error: glXCreateGLXPbufferSGIX() failed.\n");
         return false;
     }
     
     if(!_hGLContext)
     {
-        // Try indirect
-        _hGLContext = glXCreateContext(_pDisplay, visInfo, 
-                                       _bShareObjects ? context : NULL, False);
-        if ( !_hGLContext )
-        {
-            fprintf(stderr, 
-                    "RenderTexture Error: glXCreateContext() failed.\n");
-            return false;
-        }
+        fprintf(stderr,
+                "RenderTexture Error: unable to create a context for PBuffer.\n");
+        return false;
     }
 
     // [Florian] ATI returns 0 for this. There is no reason for width and
@@ -968,6 +973,9 @@ bool RenderTexture::BindBuffer( int iBuffer )
             _bIsBufferBound = true;
             _iCurrentBoundBuffer = iBuffer;
         }
+#else
+        // Not implemented for linux. No need to warn though.
+        (void)iBuffer;
 #endif
     }    
     return true;
@@ -1434,8 +1442,12 @@ void RenderTexture::_ParseModeString(const char *modeString,
             pbAttribs.push_back(true);
         }
 
-#elif defined(DEBUG) || defined(_DEBUG)
+#else
+#if defined(DEBUG) || defined(_DEBUG)
         printf("RenderTexture Error: Render to Texture not supported\n");
+        // shut up the compiler about unused variable in this case
+#endif
+        (void)pbAttribs;
 #endif  
     }
 
@@ -1777,8 +1789,9 @@ bool RenderTexture::_InitializeTextures()
 
         if (RT_COPY_TO_TEXTURE == _eUpdateMode)
         {
-            GLuint iInternalFormat;
-            GLuint iFormat;
+            // [Florian] just initialize to shut up the compiler
+            GLuint iInternalFormat = GL_RGBA8;
+            GLuint iFormat = GL_RGBA;
 
             if (_bFloat)
             {                             
