@@ -1,5 +1,5 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2011, Florian Kirsch,
+// Copyright (C) 2002-2014, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
 // This library is free software; you can redistribute it and/or 
@@ -15,7 +15,7 @@
 //
 // You should have received a copy of the GNU General Public License 
 // along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //
 // channelManager.cpp
@@ -30,6 +30,7 @@
 #endif
 
 #include "channelManager.h"
+#include "context.h"
 #include "offscreenBuffer.h"
 #include "openglHelper.h"
 #include "settings.h"
@@ -37,9 +38,8 @@
 
 namespace OpenCSG {
 
-    OpenCSG::OpenGL::OffscreenBuffer* ChannelManager::gOffscreenBuffer = 0;
-                                 bool ChannelManager::gInUse = false;
-    
+    bool ChannelManager::gInUse = false;
+
     namespace {
 
         GLint FaceOrientation = GL_CCW;
@@ -55,7 +55,7 @@ namespace OpenCSG {
 
         void defaults() {
             glViewport(OpenGL::canvasPos[0], OpenGL::canvasPos[1], OpenGL::canvasPos[2], OpenGL::canvasPos[3]);
-            glClearColor(0.0, 0.0, 0.0, 0.0);
+            glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
             glClearDepth(1.0);
             glClearStencil(0);
             glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
@@ -95,10 +95,10 @@ namespace OpenCSG {
 
     } // unnamed namespace
 
-    ChannelManager::ChannelManager() {
-
+    ChannelManager::ChannelManager()
+      : mOffscreenBuffer(0)
+    {
         glPushAttrib(GL_ALL_ATTRIB_BITS);
-        glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
         glDisable(GL_LIGHTING);
         glDisable(GL_TEXTURE_1D);
         glDisable(GL_TEXTURE_2D);
@@ -170,9 +170,9 @@ namespace OpenCSG {
             }
         }
 
-        gOffscreenBuffer = OpenGL::getOffscreenBuffer(newOffscreenType);
+        mOffscreenBuffer = OpenGL::getOffscreenBuffer(newOffscreenType);
 
-        if (!gOffscreenBuffer) {
+        if (!mOffscreenBuffer) {
             // Creating the offscreen buffer failed, maybe the OpenGL extension
             // for the specific offscreen buffer type is not supported
             return false;
@@ -221,8 +221,8 @@ namespace OpenCSG {
 
         bool rebuild = false;
 
-        if (!gOffscreenBuffer->IsInitialized()) {
-            if (!gOffscreenBuffer->Initialize(sizeX.getMax(), sizeY.getMax(), true, false)) {
+        if (!mOffscreenBuffer->IsInitialized()) {
+            if (!mOffscreenBuffer->Initialize(sizeX.getMax(), sizeY.getMax(), true, false)) {
                 // Initializing the offscreen buffer failed, maybe the OpenGL extension
                 // for the specific offscreen buffer type is not supported
                 return false;
@@ -231,10 +231,10 @@ namespace OpenCSG {
         }
         // tx == ty == 0 happens if the window is minimized, in this case don't touch a thing
         else if (tx != 0 && ty != 0) {
-            if (   gOffscreenBuffer->GetWidth() != sizeX.getMax()
-                || gOffscreenBuffer->GetHeight() != sizeY.getMax()
+            if (   mOffscreenBuffer->GetWidth() != sizeX.getMax()
+                || mOffscreenBuffer->GetHeight() != sizeY.getMax()
             ) {
-                if (!gOffscreenBuffer->Resize(sizeX.getMax(), sizeY.getMax())) {
+                if (!mOffscreenBuffer->Resize(sizeX.getMax(), sizeY.getMax())) {
                     // Resizing the offscreen buffer failed, maybe the OpenGL extension
                     // for the specific offscreen buffer type is not supported. More 
                     // likely this is a programming error in Resize(). 
@@ -246,13 +246,13 @@ namespace OpenCSG {
 
         if (rebuild) {
             // assert(gOffscreenBuffer->HasStencil());
-            gOffscreenBuffer->BeginCapture();
+            mOffscreenBuffer->BeginCapture();
             defaults();
             glGetIntegerv(GL_STENCIL_BITS, &OpenGL::stencilBits);
             OpenGL::stencilMax = 1 << OpenGL::stencilBits;
             OpenGL::stencilMask = OpenGL::stencilMax - 1;
-            gOffscreenBuffer->EndCapture();
-            gOffscreenBuffer->Bind();
+            mOffscreenBuffer->EndCapture();
+            mOffscreenBuffer->Bind();
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
         }
@@ -292,8 +292,8 @@ namespace OpenCSG {
 
     Channel ChannelManager::request() {
         if (!mInOffscreenBuffer) {
-            gOffscreenBuffer->BeginCapture();
-            if (gOffscreenBuffer->haveSeparateContext()) {
+            mOffscreenBuffer->BeginCapture();
+            if (mOffscreenBuffer->haveSeparateContext()) {
                 glFrontFace(FaceOrientation);
             }
 
@@ -303,7 +303,7 @@ namespace OpenCSG {
             mOccupiedChannels = NoChannel;
         }
 
-        if (gOffscreenBuffer->haveSeparateContext()) {
+        if (mOffscreenBuffer->haveSeparateContext()) {
             glViewport(OpenGL::canvasPos[0], OpenGL::canvasPos[1], OpenGL::canvasPos[2], OpenGL::canvasPos[3]);
             glMatrixMode(GL_PROJECTION);
             glLoadMatrixf(OpenGL::projection);
@@ -327,23 +327,23 @@ namespace OpenCSG {
 
         if ((mOccupiedChannels & Alpha) != 0) {
             result.push_back(Alpha);
-        }        
+        }
         if ((mOccupiedChannels & Red) != 0) {
             result.push_back(Red);
-        }        
+        }
         if ((mOccupiedChannels & Green) != 0) {
             result.push_back(Green);
-        }        
+        }
         if ((mOccupiedChannels & Blue) != 0) {
             result.push_back(Blue);
         }
-     
+
         return result;
     }
 
     void ChannelManager::free() {
         if (mInOffscreenBuffer) {
-            gOffscreenBuffer->EndCapture();
+            mOffscreenBuffer->EndCapture();
             mInOffscreenBuffer = false;
         }
 
@@ -362,13 +362,16 @@ namespace OpenCSG {
                     break;
                 case Blue:
                     glColorMask(GL_FALSE, GL_FALSE, GL_TRUE, GL_FALSE);
-                    break;        
+                    break;
                 case Green:
                     glColorMask(GL_FALSE, GL_TRUE, GL_FALSE, GL_FALSE);
-                    break;         
+                    break;
                 case Red:
                     glColorMask(GL_TRUE, GL_FALSE, GL_FALSE, GL_FALSE);
                     break;
+                case AllChannels:
+                    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+                    break;
                 }
             }
         else {
@@ -378,13 +381,13 @@ namespace OpenCSG {
 
 
     void ChannelManager::setupProjectiveTexture() {
-        static float splane[4] = { 1.0, 0.0, 0.0, 0.0 };
-        static float tplane[4] = { 0.0, 1.0, 0.0, 0.0 };
-        static float rplane[4] = { 0.0, 0.0, 1.0, 0.0 };
-        static float qplane[4] = { 0.0, 0.0, 0.0, 1.0 };
+        static float splane[4] = { 1.0f, 0.0f, 0.0f, 0.0f };
+        static float tplane[4] = { 0.0f, 1.0f, 0.0f, 0.0f };
+        static float rplane[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
+        static float qplane[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
 
-        gOffscreenBuffer->Bind();
-        gOffscreenBuffer->EnableTextureTarget();
+        mOffscreenBuffer->Bind();
+        mOffscreenBuffer->EnableTextureTarget();
 
         glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
         glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
@@ -416,22 +419,22 @@ namespace OpenCSG {
         // Therefore we do not check for the extension, but simply for the texture format
         // Update (08.04.2004): Fixed the flaw, but kept checking the texture format.
         // Actually that seems safer, since it should work always
-        if (gOffscreenBuffer->GetTextureTarget() == GL_TEXTURE_2D) {
+        if (!isRectangularTexture()) {
             // with ordinary pow-of-two texture coordinates are between 0 and 1
             // but we must assure only the used part of the texture is taken.
-            factorX /= static_cast<float>(gOffscreenBuffer->GetWidth());
-            factorY /= static_cast<float>(gOffscreenBuffer->GetHeight());
+            factorX /= static_cast<float>(mOffscreenBuffer->GetWidth());
+            factorY /= static_cast<float>(mOffscreenBuffer->GetHeight());
         }
 
-        float   texCorrect[16] = { factorX, 0.0, 0.0, 0.0, 
-                                   0.0, factorY, 0.0, 0.0, 
-                                   0.0,     0.0, 1.0, 0.0,
-                                   0.0,     0.0, 0.0, 1.0 };
+        float   texCorrect[16] = { factorX, 0.0f, 0.0f, 0.0f, 
+                                   0.0f, factorY, 0.0f, 0.0f, 
+                                   0.0f,    0.0f, 1.0f, 0.0f,
+                                   0.0f,    0.0f, 0.0f, 1.0f };
         
-        static float p2ndc[16] = { 0.5, 0.0, 0.0, 0.0, 
-                                   0.0, 0.5, 0.0, 0.0, 
-                                   0.0, 0.0, 0.5, 0.0, 
-                                   0.5, 0.5, 0.5, 1.0 };
+        static float p2ndc[16] = { 0.5f, 0.0f, 0.0f, 0.0f, 
+                                   0.0f, 0.5f, 0.0f, 0.0f, 
+                                   0.0f, 0.0f, 0.5f, 0.0f, 
+                                   0.5f, 0.5f, 0.5f, 1.0f };
         glPushMatrix();
         glLoadMatrixf(texCorrect);
         glMultMatrixf(p2ndc);
@@ -441,7 +444,7 @@ namespace OpenCSG {
     }
 
     void ChannelManager::resetProjectiveTexture() {
-        if (!gOffscreenBuffer->haveSeparateContext()) {
+        if (!mOffscreenBuffer->haveSeparateContext()) {
             glDisable(GL_TEXTURE_GEN_S);
             glDisable(GL_TEXTURE_GEN_T);
             glDisable(GL_TEXTURE_GEN_R);
@@ -452,7 +455,7 @@ namespace OpenCSG {
         glPopMatrix();
         glMatrixMode(GL_MODELVIEW);
 
-        gOffscreenBuffer->DisableTextureTarget();
+        mOffscreenBuffer->DisableTextureTarget();
     }
 
     void ChannelManager::setupTexEnv(Channel channel) {
@@ -464,13 +467,13 @@ namespace OpenCSG {
             if (GLEW_ARB_texture_env_dot3) {
                 switch (channel) {
                 case Red: 
-                    glColor3f(1.0, 0.5, 0.5); 
+                    glColor3f(1.0f, 0.5f, 0.5f); 
                     break;                
                 case Green: 
-                    glColor3f(0.5, 1.0, 0.5); 
+                    glColor3f(0.5f, 1.0f, 0.5f); 
                     break;
                 case Blue: 
-                    glColor3f(0.5, 0.5, 1.0); 
+                    glColor3f(0.5f, 0.5f, 1.0f); 
                     break;
                 default:
                     // should not happen!
@@ -490,13 +493,17 @@ namespace OpenCSG {
         }
     }
 
+    bool ChannelManager::isRectangularTexture() const
+    {
+        return mOffscreenBuffer->GetTextureTarget() != GL_TEXTURE_2D;
+    }
 
 
 
 
     ChannelManagerForBatches::ChannelManagerForBatches() : 
         ChannelManager(), 
-        mPrimitives(std::vector<std::pair<std::vector<Primitive*>, int> >(Blue + 1)) {
+        mPrimitives(std::vector<std::pair<std::vector<Primitive*>, int> >(AllChannels + 1)) {
     }
 
     void ChannelManagerForBatches::store(Channel channel, const std::vector<Primitive*>& primitives, int layer) {
@@ -512,7 +519,7 @@ namespace OpenCSG {
     }
 
     void ChannelManagerForBatches::clear() {
-        mPrimitives = std::vector<std::pair<std::vector<Primitive*>, int> >(Blue + 1);
+        mPrimitives = std::vector<std::pair<std::vector<Primitive*>, int> >(AllChannels + 1);
     }
 
 } // namespace OpenCSG
