@@ -1,6 +1,5 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2011, Florian Kirsch,
-// Hasso-Plattner-Institute at the University of Potsdam, Germany
+// Copyright (C) 2006-2014, Florian Kirsch
 //
 // This library is free software; you can redistribute it and/or 
 // modify it under the terms of the GNU General Public License, 
@@ -15,7 +14,7 @@
 //
 // You should have received a copy of the GNU General Public License 
 // along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //
 // context.h 
@@ -28,6 +27,7 @@
 
 #include "opencsgConfig.h"
 #include <opencsg.h>
+#include <GL/glew.h>
 
 namespace OpenCSG {
 
@@ -38,6 +38,26 @@ namespace OpenCSG {
     /// redeclared from opencsg.h
     void freeResources();
 
+    namespace OpenGL {
+
+        class OffscreenBuffer;
+
+        /// Checks the OpenGL-extensions resp. the current settings
+        /// and returns a concrete offscreen buffer, for the currently
+        /// active context in OpenCSG.
+        OffscreenBuffer* getOffscreenBuffer(OffscreenType type);
+
+        /// Given a constant(!) ARB fragment program string and its length,
+        /// returns a ARB fragment program object, for the currently
+        /// active context in OpenCSG.
+        GLuint getARBFragmentProgram(const char* prog, int len);
+
+        /// Frees all resources (offscreen buffers, fragment programs...)
+        /// allocated for the currently active context in OpenCSG.
+        void freeResources();
+
+    } // namespace OpenGL
+
 } // namespace OpenCSG
 
 #endif // __OpenCSG__context_h__
