@@ -1,5 +1,5 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2006-2011, Florian Kirsch
+// Copyright (C) 2006-2014, Florian Kirsch
 //
 // This library is free software; you can redistribute it and/or 
 // modify it under the terms of the GNU General Public License, 
@@ -14,7 +14,7 @@
 //
 // You should have received a copy of the GNU General Public License 
 // along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //
 // offscreenBuffer.h 
@@ -78,16 +78,6 @@ namespace OpenCSG {
             /// the rendering context of the caller or not.
             virtual bool haveSeparateContext() const = 0;
         };
-
-        /// Checks the OpenGL-extensions resp. the current settings
-        /// and returns a concrete offscreen buffer, for the currently
-        /// active context in OpenCSG.
-        OffscreenBuffer* getOffscreenBuffer(OffscreenType type);
-
-        /// Frees all offscreen buffers allocated for the currently
-        /// active context in OpenCSG.
-        void freeResources();
-
     } // namespace OpenGL
 
 } // namespace OpenCSG
