@@ -1,5 +1,5 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2011, Florian Kirsch,
+// Copyright (C) 2002-2014, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
 // This library is free software; you can redistribute it and/or 
@@ -15,7 +15,7 @@
 //
 // You should have received a copy of the GNU General Public License 
 // along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //
 // openglHelper.h 
@@ -34,8 +34,8 @@ namespace OpenCSG {
         GLint canvasPos[4];
 
         GLint stencilBits = 0;
-        int stencilMax = 0;
-        int stencilMask = 0;
+        unsigned int stencilMax = 0;
+        unsigned int stencilMask = 0;
 
         GLint scissorPos[4];
 
@@ -54,12 +54,24 @@ namespace OpenCSG {
             glPushMatrix();
             glLoadIdentity();
 
-            glBegin(GL_TRIANGLE_STRIP);
-                glVertex2f(-1.0, -1.0);
-                glVertex2f( 1.0, -1.0);
-                glVertex2f(-1.0,  1.0);
-                glVertex2f( 1.0,  1.0);
-            glEnd();
+            const GLfloat v[8] = {
+                  -1.0f, -1.0f,
+                   1.0f, -1.0f,
+                  -1.0f,  1.0f,
+                   1.0f,  1.0f
+            };
+
+            GLboolean origVertexArrayState = glIsEnabled(GL_VERTEX_ARRAY);
+            if (!origVertexArrayState) {
+                glEnableClientState(GL_VERTEX_ARRAY);
+            }
+
+            glVertexPointer(2, GL_FLOAT, 0, v);
+            glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+
+            if (!origVertexArrayState) {
+                glDisableClientState(GL_VERTEX_ARRAY);
+            }
 
             glMatrixMode(GL_PROJECTION);
             glPopMatrix();
