@@ -1,5 +1,5 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2011, Florian Kirsch,
+// Copyright (C) 2002-2014, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
 // This library is free software; you can redistribute it and/or 
@@ -15,7 +15,7 @@
 //
 // You should have received a copy of the GNU General Public License 
 // along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //
 // openglHelper.h 
@@ -43,9 +43,9 @@ namespace OpenCSG {
 
         extern int stencilBits;
             // number of stencil bits in the pbuffer
-        extern int stencilMax;
+        extern unsigned int stencilMax;
             // the number where the stencil value would "wrap around" to zero
-        extern int stencilMask;
+        extern unsigned int stencilMask;
             // stencilMax - 1
 
         extern int scissorPos[4];
